/*
 * Created on 26.03.2005
 *
 */
package de.wfn.builduserlibs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author riro
 *
 */
public class FileTools
{
	private static Log log = LogFactory.getLog(FileTools.class);

	private static byte[] array1 = new byte[65535];

	private static byte[] array2 = new byte[65535];

	/**
	 * Binary compare of two files for equality.
	 * @param firstFile
	 * @param inputStream 
	 * @return true if both files are equal else false;
	 * @throws IOException
	 */
	public static boolean compareFiles(File firstFile, InputStream inputStream) throws IOException
	{
		InputStream i1 = null;
		try
		{
			// read and compare bytes pair-wise
			i1 = new FileInputStream(firstFile);
			return compareFile(i1, inputStream);
		}
		finally
		{
			i1.close();
			inputStream.close();
		}
	}

	/**
	 * Binary compare of two files for equality.
	 * @param firstFile
	 * @param secondFile
	 * @return true if both files are equal else false;
	 * @throws IOException
	 */
	public static boolean compareFiles(File firstFile, File secondFile) throws IOException
	{
		if (firstFile.length() != secondFile.length())
		{
			return false;
		}
		InputStream i1 = null;
		InputStream i2 = null;
		try
		{
			i1 = new FileInputStream(firstFile);
			i2 = new FileInputStream(secondFile);
			return compareFile(i1, i2);
		}
		finally
		{
			i1.close();
			i2.close();
		}
	}

	/**
	 * Binary compare of two input streams for equality.
	 * @param inputStream1
	 * @param inputStream2
	 * @return true if both files are equal else false;
	 * @throws IOException
	 */
	public static boolean compareFile(InputStream inputStream1, InputStream inputStream2) throws IOException
	{
		long startTime = System.currentTimeMillis();
		try
		{
			int b1, b2;
			do
			{
				b1 = inputStream1.read(array1);
				b2 = inputStream2.read(array2);
			}
			while (b1 == b2 && b1 != -1 && Arrays.equals(array1, array2));

			// true only if end of file is reached
			return b1 == -1;
		}
		finally
		{
			long endTime = System.currentTimeMillis();
			if (endTime - startTime > 50)
			{
				log.info("Compare needed " + (endTime - startTime) + " milliseconds.");
			}
		}

	}

	/**
	 * Converts a package name into a path.
	 * @param packageName
	 * @return path representing the package.
	 */
	public static String packageName2Path(String packageName)
	{
		return packageName.replace('.', File.separatorChar);
	}

	/**
	 * Converts a package name into a path.
	 * @param packageName
	 * @return path representing the package.
	 */
	public static String path2PackageName(String path)
	{
		String ret = path.replace('\\', '.');
		return ret.replace('/', '.');
	}

	/**
	 * Get the file name from a (dos/unix) path.
	 * @param path
	 * @return
	 */
	public static String filenameOfPath(String path)
	{
		int pos = path.lastIndexOf('/');
		if (pos == -1)
		{
			pos = path.lastIndexOf('\\');
		}
		if (pos < path.lastIndexOf('\\'))
		{
			pos = path.lastIndexOf('\\');
		}
		if (pos == -1) pos = 0;
		return path.substring(pos + 1, path.length());
	}

	/**
	 * Returns the path of a zipentry. 
	 * @param zipEntry
	 * @return path or empty string of the given zipentry.
	 */
	public static String pathOf(ZipEntry zipEntry)
	{
		return pathOf(zipEntry.getName()); //0622126759
	}
	
	public static String pathOf(String path)
	{
		int lastSlash = path.lastIndexOf("\\");
		lastSlash = (lastSlash == -1) ? path.lastIndexOf("/") : lastSlash;
		if (lastSlash != -1)
		{
			return path.substring(0, lastSlash);
		}
		return "";
	}
	
	
	/**
	 * Copies the content from InputStream to OutputStream and closes 
	 * both streams when ready.
	 * @param in
	 * @param out
	 * @throws IOException
	 */
	public static void copyFileContent(InputStream in, OutputStream out) throws IOException
	{
		long startTime = System.currentTimeMillis();
		try
		{
			// Transfer bytes from the ZIP file to the output file
			int len;
			while ((len = in.read(array1)) > 0)
			{
				out.write(array1, 0, len);
			}
		}
		finally
		{
			long endTime = System.currentTimeMillis();
			if (endTime - startTime > 50)
			{
				log.info("Copy needed " + (endTime - startTime) + " milliseconds.");
			}
			// Close the streams
			out.close();
			in.close();
		}
	}

	/**
	 * Extract a zip entry to a temporary file
	 * @param zipFile
	 * @param zipEntry
	 * @return the temporary file containing the data of the zip entry.
	 * @throws IOException
	 */
	public static File extractToTempFile(ZipFile zipFile, ZipEntry zipEntry) throws IOException
	{
		InputStream in = zipFile.getInputStream(zipEntry);
		File newFile = File.createTempFile("USR", null);
		// Open temporary jar file.
		OutputStream out = new FileOutputStream(newFile);
		// Extract content
		FileTools.copyFileContent(in, out);
		return newFile;
	}
	
	
	/**
	 * Extract a Zipfile entry to a given directory and override the 
	 * zipentries path infomation.
	 * @param zipFile
	 * @param zipEntry
	 * @param dir
	 * @return
	 * @throws IOException
	 */
	public static File extractToDir(ZipFile zipFile, ZipEntry zipEntry, File dir) throws IOException
	{
		InputStream in = zipFile.getInputStream(zipEntry);
		if (!dir.exists())
		{
			dir.mkdirs();
		}
		File newFile = new File(dir.getAbsolutePath() + File.separatorChar + FileTools.filenameOfPath(zipEntry.getName()));
		OutputStream out = new FileOutputStream(newFile);
		// Extract content
		FileTools.copyFileContent(in, out);
		return newFile;
	}

	
	public static Vector readFileIntoStringVector(File file) throws IOException
	{
		BufferedReader br = null;
		String line;
		Vector vec = new Vector();
		try
		{
			br = new BufferedReader(new FileReader(file));
			while ((line = br.readLine()) != null)
			{
				vec.add(line);
			}
		}
		finally
		{
			if (br != null) br.close();
		}
		return vec;
	}

}
