/*
 * Created on 07.04.2005
 *
 */
package de.wfn.builduserlibs;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.util.zip.ZipException;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;

/**
 * Scan a directory hierachy for zip files. Group the zip files under
 * common names. Common names are retrieved from zip file names by removing
 * certain prefixes and postfixes.
 * @author riro
 */
public class InitialUserLibScanner implements IRunnableWithProgress
{
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(InitialUserLibScanner.class);
	
	/**
	 * Map between user lib names and their UserLib objects.
	 */
	private HashMap m_userLibs;

	/**
	 * Path to the directory hierachy under which the user libs are found.
	 */
	private File m_rootPath;

	private Vector m_purgePatternsList;

	/**
	 * Initialize object which the requested path
	 */
	public InitialUserLibScanner(File rootPath)
	{
		m_rootPath = rootPath;
		m_userLibs = new HashMap();
	}

	/**
	 * Scan for zip files in a given directory hierarchy
	 * Delete a given pattern from the found zip filenames 
	 * and use this name to make a list of unique userlibs. 
	 * @param monitor
	 */ 
	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
	{
		if (logger.isDebugEnabled())
		{
			logger.debug("populate(File) - start");
		}
	
		/*
		 * Get all zip files stored in the directory hierarchy 
		 * beneath the source directory and store them as a 
		 * LibZipFile.
		 */
	    DirectoryScanner dirScan = new DirectoryScanner(".zip", monitor);
		dirScan.processDir(m_rootPath);
		Iterator it = dirScan.iterator();

		monitor.beginTask("Zip file scanning", dirScan.numberOfFiles());
	    monitor.subTask("Scanning directories for zip files ...");
	
	    if (monitor.isCanceled()) return;
	                                
	    int i = 1;
	    int max = dirScan.numberOfFiles();
		while (it.hasNext())
		{
			File file = (File) it.next();
			updateMonitorTask(monitor, max, i, file.getName());
			try
			{
				/*
				 * Encapsulate the zip file in a UserLibZipFile which
				 * carries additional information after the zip file
				 * is analysed.
				 */
				UserLibZipFile libZipFile = new UserLibZipFile(file);
	
				/* 
				 * Convert zip filename into userlib name.
				 * If userlib name already exist add the lib to the userlib.  
				 */
				String name = removeRedundantFileNameParts(libZipFile.getFileNamePart()); 
	
				/*
				 * Check if a UserLib with the extracted name already exists.
				 * If not create a new one. Add the zip file to the UserLib.
				 * Zip file can be rearranged if they are under the wrong UserLib. 
				 */
				UserLib userLib = (UserLib) m_userLibs.get(name);
				if (userLib == null)
				{
					logger.info("New UserLib = " + name);
					userLib = new UserLib(name);
					m_userLibs.put(name, userLib);
				}
				userLib.addZipFile(libZipFile);
			}
			catch (ZipException e)
			{
				logger.error("Can't process " + file.getAbsolutePath(), e);
			}
			catch (IOException e)
			{
				logger.error("Can't process " + file.getAbsolutePath(), e);
			}
		    monitor.worked(1);
		    if (monitor.isCanceled()) return;
		    i++;
		}
	    monitor.done();
		
		if (logger.isDebugEnabled())
		{
			logger.debug("populate(File) - end");
		}
	}

	private void updateMonitorTask(IProgressMonitor monitor, int max, int i, String name)
	{
		StringBuffer buf = new StringBuffer(256);
		
		buf.append("Found ");
		buf.append(max);
		buf.append(" zip files. Nowscanning them ... ");
		buf.append(name);
		buf.append(" (");
		buf.append(i);
		buf.append("/");
		buf.append(max);
		buf.append(")");
	}

	private String removeRedundantFileNameParts(String name)
	{
		String patterns[] = readFilenamePurgePatterns();
		
		for (int i = 0; patterns != null && i < patterns.length; i++)
		{
			name = name.replaceAll(patterns[i], "");
		}
		
//		name = name.replaceAll("[\\-]*src[\\-]*", "");
//		name = name.replaceAll("[\\-]*example[s]*[\\-]*", "");
//		name = name.replaceAll("\\.zip", "");
		return name;
	}

	public HashMap getUserLibs()
	{
		return m_userLibs;
	}
	
	private String[] readFilenamePurgePatterns()
	{
		if (m_purgePatternsList == null)
		{
			m_purgePatternsList = new Vector();

			String sizeStr = BuildUserLibsPlugin.getResourceString("filenamepurgepattern.size"); //$NON-NLS-1$
			int size = 0;
			try
			{
				size = Integer.parseInt(sizeStr);
			}
			catch (RuntimeException e)
			{
				return null;
			}
			for (int i = 0; i < size; i++)
			{
				m_purgePatternsList.add(BuildUserLibsPlugin.getResourceString("filenamepurgepattern.text" + i));
			}
		}
		return (String[]) m_purgePatternsList.toArray(new String[m_purgePatternsList.size()]);
	}

}
