/*
 * Created on 28.03.2005
 */
package de.wfn.builduserlibs;

import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.jdt.internal.ui.wizards.buildpaths.CPUserLibraryElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import de.wfn.builduserlibs.packageanalyse.PackageList;

/**
 * Represents an user library like it is modeled in the Eclipse IDE.
 * Defines the user library name and the zip file of which the lib
 * consists. Additionally the jar files and their class packages are
 * retrieved for the zip files and stored.
 * @author Jens-Oliver Siepmann
 */
public class UserLib implements UserLibTreeItem
{
	/**
	 * Name of the user library
	 */
	private String m_name;  
	
	/**
	 * List of zip file from which the library is build up
	 */
	private Vector m_zipFiles;

	/**
	 * Is this user library subject to processing? 
	 */
	private boolean m_activated = true;

	private boolean m_modified = false;
	
	
	/**
	 * Construct an user library with the given name.
	 * @param name of the new user library.
	 */
	public UserLib(String name)
	{
		setName(name);
	}
	
	/**
	 * Get the name of the user library
	 * @return 
	 */
	public String getName()
	{
		return m_name;
	}
	
	/**
	 * Set the name of the user library
	 * @param name
	 */
	public void setName(String name)
	{
		m_name = name;
	}
	

	/**
	 * Add a new zip file to the user library.
	 * @param zipLib
	 */
	public void addZipFile(UserLibZipFile zipLib)
	{
		try
		{
			zipLib.setUserLib(this);
			m_zipFiles.add(zipLib);
		}
		catch(NullPointerException e)
		{
			m_zipFiles = new Vector();
			m_zipFiles.add(zipLib);
		}
		m_modified = true;
		matchJar2SourceDoc();
	}

	/**
	 * Remove a zip file from UserLib.
	 * @param element
	 */
	public void removeZipFile(UserLibZipFile element)
	{
		try
		{
			m_zipFiles.remove(element);
			m_modified = true;
		}
		catch(NullPointerException e)
		{
			return;
		}
		matchJar2SourceDoc();
	}
	
	
	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#addToTree(org.eclipse.swt.widgets.Tree)
	 */
	public void addToTree(Tree tree)
	{
		TreeItem treeItem = new TreeItem(tree, SWT.NULL);
		Archive.setTreeItemText(treeItem, m_name, m_activated);
		treeItem.setData(this);
		try
		{
			Iterator it = m_zipFiles.iterator();
			while (it.hasNext())
			{
				((UserLibZipFile)it.next()).addToTree(treeItem);
			}
			Vector jars = retrieveAllArchives();
			addSubTreeToTree(jars, "Jars", treeItem);
		}
		catch(NullPointerException e)
		{
		}
	}

	/**
	 * @return
	 */
	private Vector retrieveAllArchives()
	{
		Iterator it;
		Vector jars = new Vector();
		it = m_zipFiles.iterator();
		while (it.hasNext())
		{
			UserLibZipFile zip = (UserLibZipFile)it.next();
			if (zip.isActivated())
			{
				Iterator jarIt = zip.getJarFileNames().iterator();
				while (jarIt.hasNext())
				{
					Archive jarArchive = (Archive)jarIt.next();
					if (jarArchive.isActivated())
					{
						jars.add(jarArchive);
					}
				}
			}
		}
		return jars;
	}

	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#addToTree(org.eclipse.swt.widgets.TreeItem)
	 */
	public void addToTree(TreeItem tree)
	{
		// Intentionally left empty.
	}

	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#toggleActivation()
	 */
	public boolean toggleActivation()
	{
		return m_activated = !m_activated;
	}

	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#isActivated()
	 */
	public boolean isActivated()
	{
		return m_activated;
	}

	/**
	 * @param vec
	 * @param subTreeName
	 * @param treeItem
	 */
	private void addSubTreeToTree(Vector vec, String subTreeName, TreeItem treeItem)
	{
		if (vec != null && !vec.isEmpty())
		{
			TreeItem catTreeItem = new TreeItem(treeItem, SWT.NULL);
			Archive.setTreeItemText(catTreeItem, subTreeName, isActivated());
			Iterator it = vec.iterator();
			while (it.hasNext())
			{
				Object obj = it.next();
				if (obj instanceof String)
				{
					TreeItem subTreeItem = new TreeItem(catTreeItem, SWT.NULL);
					Archive.setTreeItemText(subTreeItem, (String) obj, isActivated());
				}
				else
				{
					if (obj instanceof Archive)
					{
						((Archive) obj).addToTree(catTreeItem);
					}
				}
			}
		}
	}
	
	
	// ==================================================
	// ==================================================
	//       Function to support the Model 
	// ==================================================
	// ==================================================

	
	
	private void matchJar2SourceDoc()
	{
		// Iterate over all activated Zipfiles and collect the archives
		// class packages.
		Vector jars = new Vector();
		Vector sourceJars = new Vector();
		Vector docJars = new Vector();
		Iterator it = m_zipFiles.iterator();
		while (it.hasNext())
		{
			UserLibZipFile zip = (UserLibZipFile)it.next();
			if (zip.isActivated())
			{
				Iterator jarIt = zip.getJarFileNames().iterator();
				while (jarIt.hasNext())
				{
					Archive jarArchive = (Archive)jarIt.next();
					if (jarArchive.isActivated())
					{
						jarArchive.getArchiveName();
						if (jarArchive.getClassPackageList() != null && jarArchive.getClassPackageList().size() != 0)
						{
							jars.add(jarArchive);
						}
						if (jarArchive.getSourcePackageList() != null && jarArchive.getSourcePackageList().size() != 0)
						{
							sourceJars.add(jarArchive);
						}
						if (jarArchive.getDocPackageList() != null && jarArchive.getDocPackageList().size() != 0)
						{
							docJars.add(jarArchive);
						}
					}
				}
				sourceJars.add(zip.getJavaPackageList());
				Iterator docIt = zip.getDocPackageNames().iterator();
				while (docIt.hasNext())
					docJars.add(zip.getDocPackageList((String)docIt.next()));
			}
		}
		
		it = jars.iterator();
		while(it.hasNext())
		{
			Archive jarArchive = (Archive)it.next();
			PackageList pkgLst = jarArchive.getClassPackageList();
			float maxMatchFactor = 0;
			float match = 0;
			Archive bestMatchArchive = null;
			PackageList bestMatchPackageList = null;
			Iterator srcIt = sourceJars.iterator();
			while (srcIt.hasNext())
			{
				Object obj = srcIt.next();
				if (obj instanceof Archive)
				{
					match = pkgLst.compare(((Archive)obj).getSourcePackageList());
					if (match > maxMatchFactor)
					{
						maxMatchFactor = match;
						bestMatchArchive = (Archive)obj;
						bestMatchPackageList = null;
					}
				}
				else
				{
					match = pkgLst.compare((PackageList)obj);
					if (match > maxMatchFactor)
					{
						maxMatchFactor = match;
						bestMatchArchive = null;
						bestMatchPackageList = (PackageList)obj;
					}
				}
			}
			if (maxMatchFactor > 0.5)
			{
				if (bestMatchArchive != null)
				{
					jarArchive.setSourceURL(bestMatchArchive.getSourcePackageList().getContainerURL());
					jarArchive.setSourceMatchFactor(maxMatchFactor);
				}
				else
				{
					jarArchive.setSourceURL(bestMatchPackageList.getContainerURL());
					jarArchive.setSourceMatchFactor(maxMatchFactor);
				}
			}
			
			maxMatchFactor = 0;
			match = 0;
			bestMatchArchive = null;
			bestMatchPackageList = null;
			Iterator docIt = docJars.iterator();
			while (docIt.hasNext())
			{
				Object obj = docIt.next();
				if (obj instanceof Archive)
				{
					Iterator tempDocIt = ((Archive)obj).getDocPackageList().iterator();
					while(tempDocIt.hasNext())
					{
						match = pkgLst.compare((PackageList)tempDocIt.next());
						if (match > maxMatchFactor)
						{
							maxMatchFactor = match;
							bestMatchArchive = (Archive)obj;
							bestMatchPackageList = null;
						}
					}
				}
				else
				{
					match = pkgLst.compare((PackageList)obj);
					if (match > maxMatchFactor)
					{
						maxMatchFactor = match;
						bestMatchArchive = null;
						bestMatchPackageList = (PackageList)obj;
					}
				}
			}
			if (maxMatchFactor > 0.5)
			{
				if (bestMatchArchive != null)
				{
					jarArchive.setDocURL(bestMatchArchive.getSourcePackageList().getContainerURL());
					jarArchive.setDocMatchFactor(maxMatchFactor);
				}
				else
				{
					jarArchive.setDocURL(bestMatchPackageList.getContainerURL());
					jarArchive.setDocMatchFactor(maxMatchFactor);
				}
			}
		}
	}

	public void copyTo(File target)
	{
		try
		{
			if (m_activated)
			{
				Iterator it = m_zipFiles.iterator();
				File dir = new File(target.getAbsolutePath() + File.separatorChar + m_name);
				dir.mkdir();
				while (it.hasNext())
				{
					((UserLibZipFile)it.next()).copyTo(dir);
				}
			}
		}
		catch(NullPointerException e)
		{
		}
	}

	public void extractJarTo(File target)
	{
		try
		{
			if (m_activated)
			{
				Iterator it = m_zipFiles.iterator();
				File dir = new File(target.getAbsolutePath() + File.separatorChar + m_name);
				dir.mkdir();
				while (it.hasNext())
				{
					((UserLibZipFile)it.next()).extractJarTo(dir);
				}
			}
		}
		catch(NullPointerException e)
		{
		}
	}

	public CPUserLibraryElement registerLib(File target)
	{
		
		if (m_activated)
		{
			CPUserLibraryElement element = new CPUserLibraryElement(getName(), false, null);		

			Iterator it = m_zipFiles.iterator();
			File dir = new File(target.getAbsolutePath() + File.separatorChar + m_name);
			dir.mkdir();
			while (it.hasNext())
			{
				((UserLibZipFile)it.next()).registerLibTo(element, dir);
			}
			return element;
		}
		return null;
	}

	public void refreshTree(TreeItem tree)
	{
		TreeItem controls[] = tree.getItems();
		HashSet oldZipFiles = new HashSet();
		oldZipFiles.addAll(m_zipFiles);
		TreeItem jarItem = null;
		for (int i = 0; i < controls.length; i++)
		{
			if (!oldZipFiles.remove(controls[i].getData()))
			{
				// The TreeItem isn't anymore member of the lib
				if (!controls[i].getText().equals("Jars"))
					controls[i].dispose();
				else
					jarItem = controls[i];
			}
		}
		if (!oldZipFiles.isEmpty())
		{
			// Add new zip files to tree
			Iterator it = oldZipFiles.iterator();
			while (it.hasNext())
			{
				((UserLibZipFile)it.next()).addToTree(tree);
			}
		}
		if (jarItem != null)
		{
			controls = jarItem.getItems();
			for (int i = 0; i < controls.length; i++)
			{
				controls[i].dispose();
			}

			Iterator it = retrieveAllArchives().iterator();
			while (it.hasNext())
			{
				((Archive)it.next()).addToTree(jarItem);
			}
			
//			oldZipFiles.clear();
//			oldZipFiles.addAll(retrieveAllArchives());
//			controls = jarItem.getItems();
//			for (int i = 0; i < controls.length; i++)
//			{
//				if (!oldZipFiles.remove(controls[i].getData()))
//				{
//					// The TreeItem isn't anymore member of the lib
//					controls[i].dispose();
//				}
//			}
//			if (!oldZipFiles.isEmpty())
//			{
//				// Add new zip files to tree
//				Iterator it = oldZipFiles.iterator();
//				while (it.hasNext())
//				{
//					((Archive)it.next()).addToTree(jarItem);
//				}
//			}
			
		}
		
		
		
	}

	/**
	 * Check if the content of the user library changed.
	 * @return true if a zip file was moved from or to the library.
	 */
	public boolean needsTreeBranchRefresh()
	{
		if (!m_modified)
			return false;
		m_modified = false;
		return true;
			
	}
}
