/*
 * Created on 28.03.2005
 */
package de.wfn.builduserlibs;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;

import org.apache.log4j.Logger;
import org.eclipse.jdt.internal.ui.wizards.buildpaths.CPUserLibraryElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import de.wfn.builduserlibs.packageanalyse.PackageCentricZipFile;

/**
 * Representation of a zip file within a user library.
 * @author Jens-Oliver Siepmann
 */
public class UserLibZipFile extends PackageCentricZipFile implements UserLibTreeItem
{
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(UserLibZipFile.class);

	public static final String SUB_TYPE_CLASS_PACKAGE = "Class Packages";
	public static final String SUB_TYPE_DOC_PACKAGE = "Doc Packages";
	public static final String SUB_TYPE_SRC_PACKAGE = "Source Packages";
	
	/**
	 * Original file object of the UserLibZipFile
	 */
	private File m_file;
	
	/**
	 * Parent UserLib to which the UserLibZipFile is related.
	 */
	private UserLib m_parentUserLib;
	
	/**
	 * Signals if the UserLibZipFile is taken into account.
	 */
	private boolean m_activated = true;
	
	/**
	 * Map from jar names to the PackageList which the jar includes.
	 */
	private Vector m_archiveVec;
	
	/**
	 * New zip file of which the userlib is constructed. 
	 * @param file
	 * @throws IOException 
	 * @throws ZipException 
	 */
	public UserLibZipFile(File file) throws ZipException, IOException
	{
		super(file);
		m_file = file;
		m_archiveVec = new Vector();
		scanForArchiveFiles();
	}

	/**
	 * @return filename part of the zip file
	 */
	public String getFileNamePart()
	{
		return m_file.getName(); 
	}

	/**
	 * @return File object of the UserLibZipFile.
	 */
	public File getFile()
	{
		return m_file;
	}
	
	/**
	 * Set a new parent UserLib for this UserLibZipFile. 
	 * @param userlib
	 */
	public void setUserLib(UserLib userlib)
	{
		m_parentUserLib = userlib;
	}

	/**
	 * @return UserLib to which this UserLibZipFile is related.
	 */
	public UserLib getUserLib()
	{
		return m_parentUserLib;
	}
	
	/**
	 * Get all jar files stored in the zip file.
	 * @return vector of <code>Archive</code> objects.
	 */
	public Vector getJarFileNames()
	{
		return m_archiveVec;
	}
	
	private void scanForArchiveFiles()
	{
		try
		{
			Enumeration zipEnum = entries();
			while (zipEnum.hasMoreElements())
			{
				ZipEntry zipEntry = (ZipEntry) zipEnum.nextElement();
				if (!zipEntry.isDirectory())
				{
					if (zipEntry.getName().toLowerCase().endsWith(".jar"))
					{
						Archive archive = new Archive(this, zipEntry);
						m_archiveVec.add(archive);
					}
				}
			}
		}
		catch (IOException e)
		{
			logger.debug("Process ZIP-File = " + m_file.getAbsolutePath() + " had an exception:", e);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see de.wfn.builduserlibs.UserLibTreeItem#addToTree(org.eclipse.swt.widgets.Tree)
	 */
	public void addToTree(Tree tree)
	{
		TreeItem subTreeItem = new TreeItem(tree, SWT.NULL);
		Archive.setTreeItemText(subTreeItem, m_file.getName(), isActivated());
	}

	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#addToTree(org.eclipse.swt.widgets.TreeItem)
	 */
	public void addToTree(TreeItem treeItem)
	{
		TreeItem subTreeItem = new TreeItem(treeItem, SWT.NULL);
		Archive.setTreeItemText(subTreeItem, m_file.getName(), isActivated());
		subTreeItem.setData(this);

		//addSubTreeToTree(getClassPackageNames(), SUB_TYPE_CLASS_PACKAGE, subTreeItem);
		//addSubTreeToTree(getJavaPackagesNames(), SUB_TYPE_SRC_PACKAGE, subTreeItem);
		//addSubTreeToTree(getDocPackageNames(), SUB_TYPE_DOC_PACKAGE, subTreeItem);
		//addSubTreeToTree(m_archiveVec, "Jar", subTreeItem);
	}

	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#toggleActivation()
	 */
	public boolean toggleActivation()
	{
		return m_activated = !m_activated ;
	}
	
	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#isActivated()
	 */
	public boolean isActivated()
	{
		return m_activated && getUserLib().isActivated();
	}
	
	/**
	 * @param vec
	 * @param subTreeName
	 * @param treeItem
	 */
	private void addSubTreeToTree(Vector vec, String subTreeName, TreeItem treeItem)
	{
		if (vec != null && !vec.isEmpty())
		{
			TreeItem catTreeItem = new TreeItem(treeItem, SWT.NULL);
			Archive.setTreeItemText(catTreeItem, subTreeName, isActivated());
			Iterator it = vec.iterator();
			while (it.hasNext())
			{
				Object obj = it.next();
				if (obj instanceof String)
				{
					TreeItem subTreeItem = new TreeItem(catTreeItem, SWT.NULL);
					Archive.setTreeItemText(subTreeItem, (String) obj, isActivated());
				}
				else
				{
					if (obj instanceof Archive)
					{
						((Archive) obj).addToTree(catTreeItem);
					}
				}
			}
		}
	}

	public void copyTo(File target) 
	{
		try
		{
			if (m_activated)
			{
				File newFile = new File (target.getAbsolutePath() + File.separatorChar + getFileNamePart());
			    OutputStream out = new FileOutputStream(newFile);
			    InputStream in = new FileInputStream(m_file);
			    FileTools.copyFileContent(in, out);
			}
		}
		catch(IOException e)
		{
			logger.error("Zip file " + m_file.getAbsolutePath() + " not copied to " + target.getAbsolutePath(), e);
		}
	}

	public void extractJarTo(File target)
	{
		try
		{
			if (m_activated)
			{
				Iterator it = m_archiveVec.iterator();
				while(it.hasNext())
				{
					((Archive)it.next()).extractTo(target);
				}
			}
		}
		catch(IOException e)
		{
			logger.error("Can't extract from " + m_file.getAbsolutePath() + " a jar file to " + target.getAbsolutePath(), e);
		}
	}

	public void registerLibTo(CPUserLibraryElement element, File target)
	{
		if (m_activated)
		{
			Iterator it = m_archiveVec.iterator();
			while(it.hasNext())
			{
				((Archive)it.next()).registerTo(element, target);
			}
		}
		
	}
}
