/*
 * Created on 05.04.2005
 */
package de.wfn.builduserlibs.packageanalyse;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;


/**
 * Zip file enhanced with content analysing methods around java package. 
 * @author riro
 */
public class PackageCentricZipFile extends ZipFile
{
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(PackageCentricZipFile.class);
	private PackageList m_classPackageNames; 
	private PackageList m_javaPackageNames;
	private HashMap m_docPackageNames = new HashMap();

	public PackageCentricZipFile(File file) throws ZipException, IOException
	{
		super(file);
		m_classPackageNames = new ClassPackageList(this);
		m_javaPackageNames = new SourcePackageList(this);
		retrievePackageNames();
	}

	public PackageCentricZipFile(File file, int mode) throws IOException
	{
		super(file, mode);
		m_classPackageNames = new ClassPackageList(this);
		m_javaPackageNames = new SourcePackageList(this);
		retrievePackageNames();
	}

	public PackageCentricZipFile(String name) throws IOException
	{
		super(name);
		m_classPackageNames = new ClassPackageList(this);
		m_javaPackageNames = new SourcePackageList(this);
		retrievePackageNames();
	}

	/**
	 * @return Returns a list of package names of the archive's class files.
	 */
	public Vector getClassPackageNames()
	{
		return m_classPackageNames.toVector();
	}
	
	/**
	 * @return Returns the archive's class files PackageList.
	 */
	public PackageList getClassPackageList()
	{
		return m_classPackageNames;
	}

	/**
	 * @return Returns a list of paths of package-list files of the archive.
	 */
	public Vector getDocPackageNames()
	{
		return new Vector(Arrays.asList(m_docPackageNames.keySet().toArray()));
	}

	/**
	 * Get the PackageList for a given package-list file.
	 * @param key
	 * @return PackageList of a javadoc's package-list files.
	 */
	public PackageList getDocPackageList(String key)
	{
		return (PackageList) m_docPackageNames.get(key);
	}
	
	/**
	 * @return Returns a list of packages names of the archive's java files.
	 */
	public Vector getJavaPackagesNames()
	{
		return m_javaPackageNames.toVector();
	}

	/**
	 * @return Returns the archive's java files PackageList.
	 */
	public PackageList getJavaPackageList()
	{
		return m_javaPackageNames;
	}
	
	/**
	 * Get all package names from doc, java and class files
	 * @param file to be analysed.
	 * @return vector of package names.
	 * @throw IOException
	 */
	private void retrievePackageNames()
	{
		Vector vec = new Vector();
		for (Enumeration zipEnum = entries(); zipEnum.hasMoreElements();)
		{
			ZipEntry zipEntry = (ZipEntry) zipEnum.nextElement();
			if (!zipEntry.isDirectory())
			{
				//vec.removeAllElements();
				String packageName = null;
				String entryName = zipEntry.getName().toLowerCase();
				try
				{
					InputStream in = getInputStream(zipEntry);
					
					packageName = CheckPackage.asJavaFile(in, entryName);
					if (packageName != null) 
					{
						m_javaPackageNames.addPackage(packageName);
						continue;
					}
					packageName = CheckPackage.asClassFile(in, entryName);
					if (packageName != null) 
					{
						m_classPackageNames.addPackage(packageName);
						continue;
					}
					vec = CheckPackage.asPackageList(in, entryName);
					if (vec != null) 
					{
						PackageList pkg = new DocPackageList(this, zipEntry);
						Iterator it = vec.iterator();
						String name = (String) it.next();
						while(it.hasNext())
							pkg.addPackage((String)it.next());
						m_docPackageNames.put(name, pkg);
						continue;
					}
				}
				catch(IOException e)
				{
					logger.error("Failed to analyse package name of " + entryName + " from stream as java file.", e);
				}

			}
		}
	}
}
